# -*- coding: utf-8 -*-

import json
import re
from typing import List

from .abstract import Bastion
from .config import validate_ip
from .exceptions import (
    CyberarkAPIException, CyberarkException, BastionException
)


class PrivilegedAccount:
    """Base class to be used with accounts fonctions"""
    def __init__(self, name: str, platformId: str, safeName: str,
                 platformAccountProperties: dict, secret: str = "", secretType:str = "Key",
                 secretManagement: dict = None,
                 remoteMachinesAccess: dict = None,
                 id: str = "",  address: str = "", userName: str = "",
                 **other):
        self.secret = secret
        if remoteMachinesAccess is None:
            remoteMachinesAccess = {"remoteMachines": "string", "accessRestrictedToRemoteMachines": True}
        if secretManagement is None:
            secretManagement = {"automaticManagementEnabled": True, "manualManagementReason": ""}
        self.remoteMachinesAccess = remoteMachinesAccess
        self.secretManagement = secretManagement
        self.secretType = secretType
        self.platformAccountProperties = platformAccountProperties
        self.safeName = safeName
        self.platformId = platformId
        self.userName = userName
        self.address = address
        self.name = name
        self.id = id
         
    def get_name(self):
        return "{}-{}".format(self.address, self.userName)

    def to_json(self):
        json_object = {"id": self.id, "name": self.name, "address": self.address, "userName": self.userName,
                       "platformId": self.platformId, "safeName": self.safeName, "secret": self.secret,
                       "platformAccountProperties": self.platformAccountProperties,
                       "secretManagement": self.secretManagement}

        return json_object

    def __str__(self):
        strrepr = self.to_json()
        strrepr["secret"] = "***********"
        return str(strrepr)


class Account:
    def __init__(self, epv: Bastion):
        self.epv = epv

    def add_account_to_safe(self, account: PrivilegedAccount) -> str:
        return self.epv.handle_request("post", 'API/Accounts', data=account.to_json(), filter_func=lambda result: result["id"])

    def is_valid_username(self, name: str) -> bool:
        special_chars = "\\/.:*?\"<>|\t\r\n\x1F"
        return not (len(name) > 128 or any(c in special_chars for c in name))

    def is_valid_safename(self, name: str) -> bool:
        special_chars = "\\/.:*?\"<>|\t\r\n\x1F"
        return not (len(name) > 28 or any(c in special_chars for c in name))

    def user_exists(self, username: str):
        if self.epv.user_list is None:
            results = self.epv.handle_request("get", 'API/Users', filter_func=lambda result: result["Users"])
            self.epv.user_list = [u['username'].lower().strip() for u in results]
        return username in self.epv.user_list

    def search_account_by_ip_addr(self, account: (PrivilegedAccount, str)) -> list:
        """
        This function search accounts using IPv4 address (or with address attribute of PrivilegedAccount object)
        :param account: A PrivilegedAccount object or IPv4 valid address
        :return: A list of "PrivilegedAccount" objects
        """
        if type(account) is str:
            address = account
        elif isinstance(account, PrivilegedAccount):
            address = account.address
        else:
            raise TypeError("search account function returned : First argument of account is not valid")

        if not validate_ip(address):
            raise TypeError("The address of the object was not considered as valid IPv4 address %s", address)

        return self.search_account_by(address=address)

    def search_account(self, expression: str) -> List[PrivilegedAccount]:
        """
        This function search an account using free text search and return a list a Privileged Account objects
        :param expression: List of keywords to search for in accounts, separated by a space.
        :return: List of PrivilegedAccount objects
        """
        return self.search_account_by(expression)

    def search_account_by(self, keywords=None, username=None, address=None, safe=None,
                          platform=None, name=None) -> List[PrivilegedAccount]:
        """
        This function allow to search using one or more parameters and return list of account id
        :param keywords: free search
        :param username: username search (field "userName")
        :param address: IP adress search (field ['platformAccountProperties']['IP'])
        :param safe: search in particular safe
        :param platform: search by platform name (no space) (field "platformId")
        :param name: search by account name (field "name")
        :return: a list of PrivilegedAccounts
        """
        if keywords is None and username is None and address is None and safe is None and platform is None \
                and name is None:
            raise BastionException("Please provide at least one search topic")

        params = {}
        search = (keywords or "") + " " + (username or "") + " " + (address or "") + " " + (platform or "") + " " + \
                 (name or "")
        search = search.strip(" ")

        if search != "":
            params["search"] = search
        if safe:
            params["filter"] = "safeName eq " + safe
        params["limit"] = "1000"

        search_results = self.epv.handle_request("get", "API/Accounts", params=params, filter_func=lambda x: x['value'])

        acc_list = []
        for acc in search_results:
            if address:
                if 'IP' in acc['platformAccountProperties']:
                    if acc['platformAccountProperties']['IP'] != address:
                        continue
            if username:
                if acc['userName'] != username:
                    continue
            if platform:
                if acc['platformId'] != platform:
                    continue
            if name:
                if acc['name'] != name:
                    continue
            acc_list.append(PrivilegedAccount(**acc))
        return acc_list
    
    def connect_using_PSM(self, account_id, connection_component):
        url, head = self.epv.get_url('API/Accounts/' + account_id + '/PSMConnect')
        head["Accept"] = 'RDP'
        
        body = {"ConnectionComponent": connection_component}
        try:
            req = self.epv.session.post(url, headers=head, data=json.dumps(body), **self.epv.request_params)
        except Exception as err:
            raise CyberarkException(err)

        if req.status_code != 200:
            content = json.loads(req.content)
            raise CyberarkAPIException(req.status_code, content["ErrorCode"], content["ErrorMessage"])

        return req.content
