import json
import unittest
from unittest import TestCase
from unittest.mock import MagicMock
import requests_mock
import bastion
from bastion.accounts import PrivilegedAccount


class TestAccount(TestCase):
    TEST_URL = 'http://vault/PasswordVault/'
    head = {"Content-type": "application/json", "Authorization": "valid_token"}
    epv = bastion.EPV('./config_mock.yml')
    epv.login = MagicMock()
    privileged = PrivilegedAccount("test_account", "platform", "testSafe", address="176.171.20.224", platformAccountProperties={})

    def test_add_account_to_safe(self):
        self.epv.get_url = MagicMock(return_value=("http://vault/PasswordVault/API/Accounts", self.head))
        with requests_mock.Mocker() as m:
            results = b'{"id": "878_25"}'
            m.post("http://vault/PasswordVault/API/Accounts", json=json.loads(results), status_code=201)
            ret_id = self.epv.account.add_account_to_safe(self.privileged)

        self.assertEqual("878_25", ret_id)

    def test_is_valid_username(self):
        self.assertTrue(self.epv.account.is_valid_username("super-admin@"))
        self.assertFalse(self.epv.account.is_valid_username("\\\\\\\\"))

    def test_is_valid_safename(self):
        self.assertTrue(self.epv.account.is_valid_safename("SAFE-NAME-12554"))
        self.assertFalse(self.epv.account.is_valid_safename("\\\\\\\\"))

    def test_user_exists(self):
        url = "http://vault/PasswordVault/API/Users"
        self.epv.get_url = MagicMock(return_value=(url, self.head))
        with requests_mock.Mocker() as m:
            with open("fixtures/account_list.json", "rb") as f:
                text = f.read()
            m.get(url, json=json.loads(text), status_code=200)
            self.assertTrue(self.epv.account.user_exists("vault_user1"))
            self.assertFalse(self.epv.account.user_exists("vault_user10"))


    def test_search_account_by_ip_addr(self):
        self.epv.get_url = MagicMock(return_value=("http://vault/PasswordVault/API/Accounts?search=176.171.20.224", self.head))
        with requests_mock.Mocker() as m:
            with open("fixtures/vault_basic.json", "rb") as f:
                text = f.read()
            m.get("http://vault/PasswordVault/API/Accounts?search=176.171.20.224", json=json.loads(text), status_code=200)
            acc = self.epv.account.search_account_by_ip_addr(self.privileged)

        self.assertEqual(1, len(acc))

    def test_search_account(self):
        self.epv.get_url = MagicMock(return_value=("http://vault/PasswordVault/API/Accounts?search=SRV-D69VHM26", self.head))
        with requests_mock.Mocker() as m:
            with open("fixtures/vault_basic.json", "rb") as f:
                text = f.read()
            m.get("http://vault/PasswordVault/API/Accounts?search=SRV-D69VHM26", json=json.loads(text), status_code=200)
            acc = self.epv.account.search_account("SRV-D69VHM26")

        self.assertEqual(1, len(acc))

    def test_search_account_by(self):
        self.epv.get_url = MagicMock(return_value=("http://vault/PasswordVault/API/Accounts?search=176.171.20.224", self.head))
        with requests_mock.Mocker() as m:
            with open("fixtures/vault_basic.json", "rb") as f:
                text = f.read()
            m.get("http://vault/PasswordVault/API/Accounts?search=176.171.20.224", json=json.loads(text), status_code=200)
            acc = self.epv.account.search_account_by(address="176.171.20.224")

        self.assertEqual(1, len(acc))

    def test_connect_using_psm(self):
        self.fail()




if __name__ == '__main__':
    unittest.main()
